/******************************************************************
 ***************   Copyright  (C)  WCH  2004-2023   ***************
 ***************         Web: http://wch.cn         ***************
 ***************      Author: WCH <tech@wch.cn>     ***************
 ***************                                    ***************
 ***************         IoCHub library Demo        ***************
 ******************************************************************
 * *************           南京沁恒微电子            ***************
 * ****************************************************************/

/********************************************
 * IoCHubLibrary Demo                       *
 * 此Demo接口能使用异步方式的都会使用异步方式    *
 * 此Demo仅支持最多打开一个会话                *
 ********************************************/
package cn.wch.iochublibrarydemo;

import android.app.Dialog;
import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.Unbinder;
import cn.wch.iochublibrary.WCHIoCHubManager;
import cn.wch.iochublibrary.config.AccessMode;
import cn.wch.iochublibrary.error.IoCHubException;

public class AccessModeDialog extends Dialog {

    @BindView(R.id.sessionModelSpinner)
    Spinner sessionModelSpinner;
    @BindView(R.id.sessionSecret)
    EditText sessionSecret;
    @BindView(R.id.cancelBtn)
    Button cancel;
    @BindView(R.id.confirmBtn)
    Button confirm;

    private Unbinder unbinder;
    private Context context;

    private ArrayAdapter<String> sessionModelAdapter;
    private List<String> sessionModelList;

    public static AccessModeDialog newInstance(Context context){
        AccessModeDialog accessModeDialog = new AccessModeDialog(context);
        return accessModeDialog;
    }

    AccessModeDialog(Context context){
        super(context);
        this.context = context;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.access_mode);
        unbinder = ButterKnife.bind(this);

        initVariable();
        showView();
        setClickedListener();
    }

    private void initVariable(){
        sessionModelList = new ArrayList<>();
        sessionModelList.add("公开模式");  //0
        sessionModelList.add("受限模式");  //1
        sessionModelList.add("私有模式");  //2
        sessionModelAdapter = new ArrayAdapter<String>(context,R.layout.spinner_style, sessionModelList);
        sessionModelSpinner.setAdapter(sessionModelAdapter);
    }

    private void showView(){
        try {
            AccessMode accessMode = WCHIoCHubManager.getInstance().getAccessMode();
            sessionModelSpinner.setSelection(accessMode.getMode());
            if (accessMode.getMode() == 1){
                sessionSecret.setEnabled(true);
                sessionSecret.setText(accessMode.getSessionKey());
            } else {
                sessionSecret.setEnabled(false);
                sessionSecret.setText("");
            }
        } catch (IoCHubException e) {
            showToast("获取会话模式失败:" + e.getErrorCode());
        }
    }

    private void setClickedListener(){
        sessionModelSpinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                if (position == 1){
                    sessionSecret.setEnabled(true);
                } else {
                    sessionSecret.setEnabled(false);
                    sessionSecret.setText("");
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {
            }
        });

        cancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismiss();
            }
        });

        confirm.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                confirm();
            }
        });
    }

    private void confirm(){
        String secret = null;
        int model = sessionModelSpinner.getSelectedItemPosition();
        if (model == 1){
            int length = sessionSecret.getText().length();
            if (length == 0 || length > 8){
                showToast("输入的会话秘钥不规范");
                return;
            } else {
                secret = sessionSecret.getText().toString();
            }
        }
        AccessMode accessMode = new AccessMode(model,secret);
       int ret = WCHIoCHubManager.getInstance().setAccessMode(accessMode);
        dismiss();
    }

    @Override
    public void dismiss() {
        super.dismiss();
        unbinder.unbind();
    }

    private void showToast(String message){
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(context,message,Toast.LENGTH_SHORT).show();
            }
        });
    }
}
