/* 
 * IocHub Demo for Linux       
 *
 * Copyright (C) 2004-2023 Nanjing Qinheng Microelectronics Co., Ltd.
 * Web:      http://wch.cn
 * Author:   WCH <tech@wch.cn>
 * Version:  V1.1
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 */
#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QNetworkInterface>
#include <QMessageBox>


MainWindow *MainWindow::m_This = nullptr;

Q_DECLARE_METATYPE(IOCHUB_SESSION_EVENT_S);
Q_DECLARE_METATYPE(IOCHUB_SESSION_READ_EVENT_S);
Q_DECLARE_METATYPE(IOCHUB_SESSION_TRANSFER_CHANGE_EVENT_S);
Q_DECLARE_METATYPE(IOCHUB_SESSION_EXCEPTION_EVENT_S);
Q_DECLARE_METATYPE(IOCHUB_SERVICE_ERR_EVENT_S);

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    on_refresh_network_interface_Btn_pressed();


    int result = IoCHub_Init();
    if (result == 0) {
        IoCHub_RegisterEvent(iochub_event);
    }
    getOrSetGlobalCfg(1);
    qRegisterMetaType<session_event_t>();
    qRegisterMetaType<session_read_event_t>();
    qRegisterMetaType<session_transfer_change_event_t>();
    qRegisterMetaType<session_exception_event_t>();
    qRegisterMetaType<service_event_t>();

    /**
     * 设置状态栏
     */
    QLabel *statusBarTitleLabel = new QLabel();
    statusBarTitleLabel->setText("就绪");   //label显示内容
    statusBar()->setStyleSheet(QString("QStatusBar::item{border: 0px}")); // 设置不显示label的边框
    statusBar()->setSizeGripEnabled(false); //设置是否显示右边的大小控制点
    statusBar()->addWidget(statusBarTitleLabel);

    char *hostIp = "58.213.74.190:38089";
//    char *hostIp = "192.168.111.93:38089";
//    scannedDevList.clear();
    m_This = this;
    ui->hostEdt->setText(QByteArray(hostIp));
    ui->send_area_file_trans_btn->setEnabled(false);

    delayedSendingTimer = new QTimer(this);
    refreshSpeedTimer = new QTimer(this);
    connect(this,SIGNAL(emit_scanned_dev(LAN_SCAN_RESULT_T *, int)),this,SLOT(updateLanScanCombox(LAN_SCAN_RESULT_T *, int)));
    connect(this, SIGNAL(emit_iochub_event(int, void *)), this, SLOT(slot_iochub_event(int, void *)));
    connect(delayedSendingTimer, SIGNAL(timeout()), this, SLOT(on_delayed_timer_sending()));
    connect(refreshSpeedTimer, SIGNAL(timeout()), this, SLOT(refresh_ui_speed()));
    refreshSpeedTimer->start(500);

    sendFileSpeedTimer = new QTimer(this);
    connect(sendFileSpeedTimer, SIGNAL(timeout()), this, SLOT(slot_sending_file()));
}

MainWindow::~MainWindow()
{
    refreshSpeedTimer->stop();
    refreshSpeedTimer->deleteLater();
    delete refreshSpeedTimer;
    refreshSpeedTimer = nullptr;

    delayedSendingTimer->stop();
    delayedSendingTimer->deleteLater();
    delete delayedSendingTimer;
    delayedSendingTimer = nullptr;
    if (savingFile != nullptr && savingFile->isOpen()) {
        savingFile->close();
    }
    if (localFile != nullptr && localFile->isOpen()) {
        localFile->close();
    }
    if (sendFileSpeedTimer->isActive()) {
        sendFileSpeedTimer->stop();
    }
    sendFileSpeedTimer->deleteLater();
    delete sendFileSpeedTimer;
    sendFileSpeedTimer = nullptr;

    delete ui;
}

QString MainWindow::size_transfer(uint64_t size) {
    QStringList list;
    list << "KB/S" << "MB/S" << "GB/S" << "TB/S";
    double transferSize = size * 1.0;
    QStringListIterator i(list);
    QString unit("B/S");

    while(transferSize >= 1024.0 && i.hasNext())
    {
     unit = i.next();
     transferSize /= 1024.0;
    }
    return QString().setNum(transferSize, 'f', 2)+unit;
}

void MainWindow::on_refresh_network_interface_Btn_pressed()
{
    ui->local_network_interface_comboBox->clear();
    QList<QNetworkInterface> ifaceList = QNetworkInterface::allInterfaces();
    for (int var = 0; var < ifaceList.count(); var++) {
        QNetworkInterface item = ifaceList.at(var);
        if (!item.flags().testFlag(QNetworkInterface::IsLoopBack)) {
            QString strIpAddr = "";
            for (int j = 0; j < item.addressEntries().size(); j++) {
                if (item.addressEntries().at(j).ip() != QHostAddress::LocalHost && item.addressEntries().at(j).ip().protocol() == QAbstractSocket::IPv4Protocol) {
                    strIpAddr = item.addressEntries().at(j).ip().toString();
                    ui->local_network_interface_comboBox->addItem(QString("%1(%2)").arg(item.humanReadableName()).arg(strIpAddr));
                }
            }
        }

    }
    char * localNodeID = (char *)malloc(20);
    int length = 0;
    IoCHub_GetLocalID(localNodeID, &length);
    if (length > 0) {
        ui->local_nodeId_lineEdit->setText(QString(QByteArray(localNodeID, length)));
    }
    free(localNodeID);
}

void MainWindow::on_start_service_Btn_pressed()
{
    if (startService) {
        /*关闭*/
        IoCHub_Stop();
        startService = false;
        ui->start_service_Btn->setText(QString("启动服务"));
        ui->nat_type_lineEdit->setText("");
    } else {
        if (ui->hostEdt->text().isEmpty()) {
            QMessageBox::warning(this, "错误", "目标服务器地址不能为空");
            return;
        }
        QString ipAddr = ui->hostEdt->text();
        QByteArray temp = ipAddr.toLocal8Bit();
        char * hostIp = temp.data();
        int result = IoCHub_Start(hostIp, 5000, true);
        if (result != 0) {
            QMessageBox::warning(this, "错误", QString("启动服务失败，错误码：%1").arg(result));
            return;
        }
        startService = true;
        ui->start_service_Btn->setText(QString("关闭服务"));
    }

}

void MainWindow::read_data(int sessionFd, int length) {
    char *read_buf;
    int realLen = 0;
    if (length > 0) {
        read_buf = (char*)malloc(length);
        realLen = Session_Read(sessionFd, read_buf, length);
        if (!ui->recved_stop_show_checkbox->isChecked()) {
            ui->recved_area_data_text->append(QByteArray(read_buf, realLen));
        }
        this->recvBytesCount += length;
        if (saveIntoFileFlag && savingFile && savingFile->isOpen()) {
            savingFile->write(read_buf, realLen);
            savingFile->flush();
        }

        free(read_buf);
        read_buf = NULL;
    }
}

void MainWindow::on_open_session_Btn_pressed()
{
    if (ui->dst_nodeId_lineEdit->text().isEmpty()) {
        QMessageBox::warning(this, "错误", QString("请输入连接端NodeID"));
        return;
    }
    if (ui->dst_nodeId_lineEdit->text().size() > 16 || ui->dst_nodeId_lineEdit->text().size() < 15) {
        QMessageBox::warning(this, "错误", QString("请输入有效的NodeID"));
        return;
    }
    if (!ui->dst_access_secret_lineEdit->text().isEmpty()) {
        if (ui->dst_access_secret_lineEdit->text().size() > 8) {
            QMessageBox::warning(this, "错误", QString("请输入有效的Secret"));
            return;
        }
    }
    QString dstNodeIdStr = ui->dst_nodeId_lineEdit->text();
    QByteArray temp = dstNodeIdStr.toLocal8Bit();
    char * dstNodeId = temp.data();
    QString secretLineStr = ui->dst_access_secret_lineEdit->text();
    QByteArray secretTemp = secretLineStr.toLocal8Bit();
    char *dstSecret = secretTemp.data();
    int result = 0;
    if (sessionOpenFlag) {
        closeSession(this->sessionFd);
    } else {
        result = Session_Open(dstNodeId, dstSecret, 5000, true);
        if (result < 0) {
            result = 0-result;
            switch (result) {
            case 300:
                QMessageBox::warning(this, "错误", QString("当前会话已存在"));
                return;
            default:
                QMessageBox::warning(this, "错误", QString("打开会话失败,错误码:%1").arg(result));
                break;
            }
        } else {
            activeOpenSession(result);
        }
    }
}

void MainWindow::activeOpenSession(int sessionFd) {
    this->sessionFd = sessionFd;
    sessionOpenFlag = true;
    int transferType = Session_GetTransferType(sessionFd);
    changeTransferType(transferType);
    ui->open_session_Btn->setText("关闭会话");
    ui->dst_nodeId_lineEdit->setEnabled(false);
    ui->dst_access_secret_lineEdit->setEnabled(false);

    ui->send_area_file_trans_btn->setEnabled(true);
}

void MainWindow::unActiveOpenSession(int sessionFd, QByteArray nodeIdStr) {
    if (sessionOpenFlag) {
        return;
    }
    activeOpenSession(sessionFd);
    ui->dst_nodeId_lineEdit->setText(nodeIdStr);
}

void MainWindow::unActiveCloseSession() {
    ui->open_session_Btn->setText("打开会话");
    ui->dst_nodeId_lineEdit->setEnabled(true);
    ui->dst_access_secret_lineEdit->setEnabled(true);
    ui->send_area_file_trans_btn->setEnabled(false);
    sessionOpenFlag = false;
    sessionFd = 0;
}

void MainWindow::closeSession(int sessionFd){
    Session_Close(sessionFd);
    unActiveCloseSession();
}

void MainWindow::changeTransferType(int transferType) {
    switch (transferType) {
    case 0x00:  //未知类型
        break;
    case 0x01:  // P2P（广域网）
        break;
    case 0x02:  // TCP（广域网）
        break;
    case 0x03:  // LAN（局域网）
        break;
    }
}

QString MainWindow::randCharToStr() {
    QString tmp = QString("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789");
    QByteArray str;
    QTime t;
    t = QTime::currentTime();
    qsrand(t.msec()+t.second()*1000);
    int max = qrand() % 100 + 20;  //字符串长度
    for(int i=0; i < max; i++){
        int len = qrand() % tmp.length();
        str.append(tmp.at(len));
    }
    return str;
}

void MainWindow::iochub_event(int eventCode, void * eventData) {
    session_event_t *session_event;
    service_event_t *err_event;
    session_read_event_t *read_event;
    session_transfer_change_event_t *transfer_change_event;
    session_exception_event_t *session_exception_event;
    switch(eventCode) {
    case EVENT_RELAY_START:
        err_event = (service_event_t *)malloc(sizeof (service_event_t));
        memcpy(err_event, (service_event_t *)eventData, sizeof (service_event_t));
        emit m_This->emit_iochub_event(eventCode, err_event);
        break;
    case EVENT_ACTIVE_SESSION_OPEN:   // 例程使用同步方式，若采用异步，则需要在此处处理
        break;
    case EVENT_UNACTIVE_SESSION_OPEN:
        session_event = (session_event_t *)malloc(sizeof (session_event_t));
        memcpy(session_event, (session_event_t *)eventData, sizeof (session_event_t));
        emit m_This->emit_iochub_event(eventCode, session_event);
        break;
    case EVENT_SESSION_CLOSE:
        session_event = (session_event_t *)malloc(sizeof (session_event_t));
        memcpy(session_event, (session_event_t *)eventData, sizeof (session_event_t));
        emit m_This->emit_iochub_event(eventCode, session_event);
        break;
    case EVENT_SESSION_READ_NOTIFY:
        read_event = (session_read_event_t *)malloc(sizeof (session_read_event_t));
        memcpy(read_event, (session_read_event_t *)eventData, sizeof (session_read_event_t));
        emit m_This->emit_iochub_event(eventCode, read_event);
        break;
    case EVENT_SESSION_TRANSFER_TYPE_CHANGE:
        transfer_change_event = (session_transfer_change_event_t *)malloc(sizeof (session_transfer_change_event_t));
        memcpy(transfer_change_event, (session_transfer_change_event_t *)eventData, sizeof (session_transfer_change_event_t));
        emit m_This->emit_iochub_event(eventCode, transfer_change_event);
        break;
    case EVENT_SESSION_EXCEPTION:
        session_exception_event = (session_exception_event_t *)malloc(sizeof (session_exception_event_t));
        memcpy(session_exception_event, (session_exception_event_t *)eventData, sizeof (session_exception_event_t));
        emit m_This->emit_iochub_event(eventCode, session_exception_event);
        break;
    case EVENT_OPERATION_ERR:
        err_event = (service_event_t *)malloc(sizeof (service_event_t));
        memcpy(err_event, (service_event_t *)eventData, sizeof (service_event_t));
        emit m_This->emit_iochub_event(eventCode, err_event);
        break;
    }

}

void MainWindow::slot_iochub_event(int eventCode, void * eventData) {
    session_event_t * session_event;
    service_event_t *err_event;
    session_read_event_t *read_event;
    session_transfer_change_event_t *transfer_change_event;
    session_exception_event_t *session_exception_event;
    switch(eventCode) {
    case EVENT_RELAY_START:

        break;
    case EVENT_ACTIVE_SESSION_OPEN:   // 例程使用同步方式，若采用异步，则需要zai在此处处理
        session_event = (session_event_t *)eventData;
        if (session_event) {

        }
        break;
    case EVENT_UNACTIVE_SESSION_OPEN:
        session_event = (session_event_t *)eventData;
        if (session_event) {
            QByteArray nodeIdStr((char *)session_event->peerNodeIdStr, 16);
            unActiveOpenSession(session_event->sessionFd, nodeIdStr);
        }
        break;
    case EVENT_SESSION_READ_NOTIFY:
        read_event = (session_read_event_t *)eventData;
        if (read_event) {
            read_data(read_event->sessionFd, read_event->min_readable_len);
        }
        break;
    case EVENT_SESSION_CLOSE:
        session_event = (session_event_t *)eventData;
        if (session_event) {
            if (session_event->sessionFd == this->sessionFd) {
                unActiveCloseSession();
            }
        }
        break;
    case EVENT_SESSION_TRANSFER_TYPE_CHANGE:
        transfer_change_event = (session_transfer_change_event_t *)eventData;
        if (transfer_change_event) {
            if (transfer_change_event->sessionFd == this->sessionFd) {
                changeTransferType(transfer_change_event->type);
            }
        }
        break;
    case EVENT_SESSION_EXCEPTION:
        session_exception_event = (session_exception_event_t *)eventData;
        if (session_exception_event) {
            if (session_exception_event->sessionFd == this->sessionFd) {
                closeSession(sessionFd);
                QMessageBox::warning(this, "错误", QString("会话异常，错误码：%1").arg(session_exception_event->errCo));
                return;
            }
        }
        break;
    case EVENT_OPERATION_ERR:
        err_event = (service_event_t *)eventData;
        if (err_event) {
             QMessageBox::warning(this, "错误", QString("异常错误码：%1").arg(err_event->errCo));
             return;
        }
        break;
    }
    free(eventData);
    eventData = NULL;
}

void MainWindow::broadcast_cb(LAN_SCAN_RESULT_T * pLanDevNodes, int nodeSize) {
    emit m_This->emit_scanned_dev(pLanDevNodes, nodeSize);
}

void MainWindow::updateLanScanCombox(LAN_SCAN_RESULT_T * pLanDevNodes, int nodeSize) {
    for (int i = 0; i < nodeSize; i++) {
        char * nodeId = pLanDevNodes->nodeID;
        LanScanDev *scanDev = new LanScanDev;
        scanDev->devNodeId = QString(QByteArray(nodeId, 16));
        scanDev->ipAddr = QString(QByteArray((char *)pLanDevNodes->addrStr));
//        scannedDevList.append(scanDev);
        ui->lan_device_list_comboBox->addItem(QString("%1").arg(scanDev->devNodeId));
        pLanDevNodes++;
        qDebug() << "nodeId-->" << scanDev->devNodeId << "---IP-->" << scanDev->ipAddr;
    }
    qDebug() << "broadcast_cb ===>" << nodeSize;
}

void MainWindow::on_lan_broadcast_Btn_pressed()
{
    ui->lan_device_list_comboBox->clear();
    int status = IoCHub_ScanLanNodes("192.168.111.93,192.168.2.101", 3000, &MainWindow::broadcast_cb);
    if (status != 0x00) {
        QMessageBox::warning(this, "错误", QString("局域网扫描失败，错误码：%1").arg(status));
        return;
    }
}

void MainWindow::on_lan_device_list_comboBox_currentIndexChanged(const QString &currentIndexStr)
{
    ui->dst_nodeId_lineEdit->setText(currentIndexStr);
}

void MainWindow::on_send_area_generate_btn_clicked()
{
    ui->send_area_data_text->clear();
    QString data = randCharToStr();
    ui->send_area_data_text->insertPlainText(data);
}

void MainWindow::on_recved_area_clear_btn_clicked()
{
    ui->recved_area_data_text->clear();
    ui->recved_area_count_lineEdit->clear();
    ui->recved_area_count_lineEdit->setText(0);
    ui->recved_area_speed_lineEdit->clear();
    ui->recved_area_speed_lineEdit->setText(0);
    this->recvSpeed = 0;
    this->recvBytesCount = 0;
    this->oldRecvBytesCount = 0;
}

void MainWindow::on_recved_stop_show_checkbox_stateChanged(int state)
{
    if (ui->recved_stop_show_checkbox->checkState() == Qt::CheckState::Checked) {
        recvedStopShowFlag = true;
    } else {
        recvedStopShowFlag = false;
    }
}

void MainWindow::on_send_area_clear_btn_clicked()
{
    ui->send_area_data_text->clear();
    ui->send_area_count_lineEdit->clear();
    ui->send_area_count_lineEdit->setText(0);
    ui->send_area_speed_lineEdit->clear();
    ui->send_area_speed_lineEdit->setText(0);
    this->sendSpeed = 0;
    this->sendBytesCount = 0;
    this->oldSendBytesCount = 0;
}

void MainWindow::handlerSendData() {
    QString sendDataText = ui->send_area_data_text->toPlainText();
    int length = sendDataText.length();
    QByteArray temp = sendDataText.toLocal8Bit();
    char * buffer = temp.data();


    int writeLen = Session_Write(sessionFd, buffer, length);
    if (writeLen >= 0) {
        sendBytesCount += writeLen;
    } else {
        writeLen = 0 - writeLen;
        if (writeLen == 401) {
            /* 缓冲区溢出 */
        }
    }
}

/**
 * 定时发送
 * @brief MainWindow::on_delayed_timer_sending
 */
void MainWindow::on_delayed_timer_sending() {
    handlerSendData();
}

/**
 * 更新UI线程
 * @brief MainWindow::refresh_ui_speed
 */
void MainWindow::refresh_ui_speed() {
     recvSpeed = (recvBytesCount - oldRecvBytesCount) * 2;
     oldRecvBytesCount = recvBytesCount;
     ui->recved_area_count_lineEdit->setText(QString("%1").arg(recvBytesCount));
     ui->recved_area_speed_lineEdit->setText(QString("%1").arg(size_transfer(recvSpeed)));

     sendSpeed = (sendBytesCount - oldSendBytesCount) * 2;
     oldSendBytesCount = sendBytesCount;
     ui->send_area_count_lineEdit->setText(QString("%1").arg(sendBytesCount));
     ui->send_area_speed_lineEdit->setText(QString("%1").arg(size_transfer(sendSpeed)));
}

/**
 * 发送按钮槽事件
 * @brief MainWindow::on_send_area_send_btn_clicked
 */
void MainWindow::on_send_area_send_btn_clicked()
{
    if (sendTimerEnableFlag) {
        if (isSendingFlag) {
            ui->send_area_send_btn->setText("发送");
            isSendingFlag = false;
            ui->send_timer_checkbox->setEnabled(true);
            ui->send_delayed_timer_timeout->setEnabled(true);
            if (delayedSendingTimer->isActive()) {
                delayedSendingTimer->stop();
            }
        } else {
            if (!delayedSendingTimer->isActive()) {
                bool ok = true;
                uint delayedTimeout = ui->send_delayed_timer_timeout->text().toUInt(&ok, 10);
                delayedSendingTimer->start(delayedTimeout);
            }
            ui->send_timer_checkbox->setEnabled(false);
            ui->send_delayed_timer_timeout->setEnabled(false);
            ui->send_area_send_btn->setText("停止发送");
            isSendingFlag = true;
        }
    } else {
        handlerSendData();
    }
}

void MainWindow::on_send_timer_checkbox_stateChanged(int state)
{
    if (ui->send_timer_checkbox->checkState() == Qt::CheckState::Checked) {
        sendTimerEnableFlag = true;
    } else {
        sendTimerEnableFlag = false;
    }
}

void MainWindow::on_access_cfg_set_Btn_pressed()
{
    int modeType = ui->access_mode_comboBox->currentIndex();
    IOCHUB_ACCESS_MODE_T * accessMode = (IOCHUB_ACCESS_MODE_T *)malloc(sizeof (IOCHUB_ACCESS_MODE_T));
    memset(accessMode, 0, sizeof (IOCHUB_ACCESS_MODE_T));
    accessMode->modeType = modeType;
    accessMode->sessionKey = (char *) malloc(10);
    if (!ui->socket_access_key_lineEdit->text().isEmpty()) {
        QString temp = ui->socket_access_key_lineEdit->text();
        if (temp.size() >= 8) {
            QMessageBox::warning(this, "提示", QString("会话密钥不能超过8个字节"));
            return;
        }

        char * tempData = (char *)temp.data();
        memcpy(accessMode->sessionKey, temp.data(), temp.size());
        accessMode->keySize = temp.size();
    }

    IoCHub_SetAccessMode(accessMode);
}

void MainWindow::on_deInit_Btn_pressed()
{
    int result = 0;
    if (deInitFlag) {

    } else {
        result = IoCHub_DeInit();
        if (result == 0) {
            deInitFlag = true;
            ui->deInit_Btn->setEnabled(false);
        }
    }

}

void MainWindow::on_send_file_sel_checkbox_clicked()
{
    if (ui->send_file_sel_checkbox->isChecked()) {
        if (openFile()) {
        } else {
            ui->send_file_sel_checkbox->setChecked(false);
        }

    } else {
        ui->statusbar->showMessage("就绪");
    }
}


void MainWindow::on_send_area_file_trans_btn_clicked() {
    if (sendingFileFlag) {
        ui->send_area_file_trans_btn->setText("发送文件");
        ui->send_area_send_btn->setEnabled(true);
        if (sendFileSpeedTimer->isActive()) {
            sendFileSpeedTimer->stop();
        }
        sendingFileFlag = false;
    } else {
        ui->send_area_file_trans_btn->setText("停止发送");
        ui->send_area_send_btn->setEnabled(false);
        localFile = new QFile(fileName);
        if(!localFile->open(QFile::ReadOnly)) {
            qDebug() << "open file error!";
            return;
        }
        //文件总大小
        fileTotalBytes = localFile->size();
        fileSendPerPackageSize = 4096;
        if (!sendFileSpeedTimer->isActive()) {
            sendFileSpeedTimer->start(50);
        }
        fileBytesWritten = 0;
        sendingFileFlag = true;
    }
}

void MainWindow::slot_sending_file() {
    char fileData[4096] = {0};
    localFile->seek(fileBytesWritten);
    int realReadLen = localFile->read(fileData, fileSendPerPackageSize);
    if (realReadLen > 0) {
        int realWrite = Session_Write(sessionFd, fileData, realReadLen);
        if (realWrite > 0) {
            fileBytesWritten += realWrite;
            sendBytesCount += realWrite;
        }
    }
    if (fileBytesWritten >= fileTotalBytes) {
        sendFileSpeedTimer->stop();
        localFile->close();
        ui->send_area_file_trans_btn->setText("发送文件");
        ui->send_area_send_btn->setEnabled(true);
         sendingFileFlag = false;
    }
}

bool MainWindow::openFile() {
    fileName = QFileDialog::getOpenFileName(this);
    if(!fileName.isEmpty()) {
        if (sessionOpenFlag) {
            ui->send_area_file_trans_btn->setEnabled(true);
        }
        ui->statusbar->showMessage(tr("打开文件 %1 成功！")
                                        .arg(fileName));
        ui->send_file_select_path->setText(fileName);
        return true;
    } else {
        ui->send_area_file_trans_btn->setEnabled(false);
        ui->send_file_select_path->setText("");
    }
    return false;
}

bool MainWindow::selectSavingFile() {
    selectedSavingFilePath = QFileDialog::getOpenFileName(this);
    if(selectedSavingFilePath.isEmpty()) {
        ui->recved_file_saved_path_lineEdit->setEnabled(true);
        ui->recved_file_saved_path_lineEdit->setText("");
        return false;
    } else {
        ui->recved_file_saved_path_lineEdit->setEnabled(false);
        ui->recved_file_saved_path_lineEdit->setText(selectedSavingFilePath);
    }
    return true;
}

void MainWindow::on_recved_file_saved_checkbox_clicked()
{
    bool flag = false;
    if (ui->recved_file_saved_checkbox->isChecked()) {
        flag = selectSavingFile();
        if (flag) {
            if (savingFile != nullptr && saveIntoFileFlag) {
                savingFile->close();
                savingFile = nullptr;
            }
            savingFile = new QFile(selectedSavingFilePath);
            if (!savingFile->open(QIODevice::Append)) {
                QMessageBox::warning(this, "提示", QString("选择保存路径文件不正确"));
                return;
            }
            saveIntoFileFlag = true;
        } else {
            ui->recved_file_saved_checkbox->setChecked(false);
        }
    } else {
        ui->recved_file_saved_path_lineEdit->setEnabled(true);
        if (saveIntoFileFlag) {
            if (savingFile != nullptr) {
                savingFile->close();
                savingFile = nullptr;
            }
            saveIntoFileFlag = false;
        }
    }
}

void MainWindow::getOrSetGlobalCfg(int type) {
    IOCHUB_INIT_CFG_T * init_cfg = (IOCHUB_INIT_CFG_T *)malloc(sizeof (IOCHUB_INIT_CFG_T));
    memset(init_cfg, 0, sizeof (IOCHUB_INIT_CFG_T));
    int result = 0;
    if (type == 1) {
        /* GET */
        IoCHub_GetCurrentCfg(init_cfg);
        ui->socket_connect_num_lineEdit->setText(QString("%1").arg(init_cfg->maxConnSize));
        ui->socket_package_size_lineEdit->setText(QString("%1").arg(init_cfg->sendSizePerPackage));
        ui->socket_connect_window_lineEdit->setText(QString("%1").arg(init_cfg->socketWndSize));
        ui->socket_connect_window_timeout_lineEdit->setText(QString("%1").arg(init_cfg->socketWndTimeout));
        if (init_cfg->supportCompress) {
            ui->data_compress_checkBox->setChecked(true);
        } else {
            ui->data_compress_checkBox->setChecked(false);
        }
        if (init_cfg->openDeviceLock) {
            ui->device_lock_checkBox->setChecked(true);
        } else {
            ui->device_lock_checkBox->setChecked(false);
        }
        ui->recv_buffer_lineEdit->setText(QString("%1").arg(init_cfg->recvBufferSize));
        ui->send_buffer_lineEdit->setText(QString("%1").arg(init_cfg->sendBufferSize));
    } else {
        /* SET */
        IoCHub_GetCurrentCfg(init_cfg);
        init_cfg->maxConnSize = ui->socket_connect_num_lineEdit->text().toUInt();
        init_cfg->sendSizePerPackage = ui->socket_package_size_lineEdit->text().toInt();
        init_cfg->socketWndSize = ui->socket_connect_window_lineEdit->text().toUInt();
        init_cfg->socketWndTimeout = ui->socket_connect_window_timeout_lineEdit->text().toUInt();
        init_cfg->supportCompress = ui->data_compress_checkBox->isChecked()?true:false;
        init_cfg->openDeviceLock = ui->device_lock_checkBox->isChecked()?true:false;
        init_cfg->recvBufferSize = ui->recv_buffer_lineEdit->text().toUInt();
        init_cfg->sendBufferSize = ui->send_buffer_lineEdit->text().toUInt();
        result = IoCHub_SetCfg(init_cfg);
        if (result != 0) {
            QMessageBox::warning(this, "提示", QString("配置参数操作失败:%1").arg(result));
        } else {
            QMessageBox::warning(this, "提示", QString("配置参数操作Success"));
        }
    }
    free(init_cfg);
    init_cfg = NULL;
}

void MainWindow::on_cfg_set_Btn_clicked() {
    getOrSetGlobalCfg(2);
}

void MainWindow::on_session_transfer_type_Btn_clicked()
{
    int type = 0;
    if (sessionOpenFlag) {
        type = Session_GetTransferType(sessionFd);
        switch (type) {
        case 0:
            ui->session_transfer_type_lineEdit->setText("未知");
            break;
        case 1:
            ui->session_transfer_type_lineEdit->setText("P2P");
            break;
        case 2:
            ui->session_transfer_type_lineEdit->setText("转发");
            break;
        case 3:
            ui->session_transfer_type_lineEdit->setText("LAN");
            break;
        case 203:
            QMessageBox::warning(this, "提示", QString("未初始化IoCHub"));
            break;
        case 400:
            QMessageBox::warning(this, "提示", QString("会话不存在"));
            break;
        }
    } else {
        QMessageBox::warning(this, "提示", QString("未打开会话连接"));
    }
}

void MainWindow::on_get_nat_type_Btn_clicked()
{
    int type = 0;
    if (startService) {
        type = IoCHub_GetNATType();
        if (type < 0) {
            type = 0 - type;
            QMessageBox::warning(this, "提示", QString("获取NAT类型操作失败:%1").arg(type));
            return;
        }
        switch (type) {
        case 0x00:
            ui->nat_type_lineEdit->setText("非对称");
            break;
        case 0x10:
            ui->nat_type_lineEdit->setText("对称");
            break;
        default:
            ui->nat_type_lineEdit->setText("未知");
            break;
        }
    }
}

