//*******************************************
//**  Copyright  (C)  WCH  2004-2023       **
//**  Web:  http://wch.cn                  **
//*******************************************
//**  iochub libray for iOS/macOS          **
//*******************************************
//
// iocHub接口库 V1.1
// 南京沁恒微电子
// 运行环境: iOS/macOS
//

#import <Foundation/Foundation.h>

#define IOCHUB_SUCCESS                          0UL         /* 操作成功 */
#define IOCHUB_REGISTERING                      100UL       /* 正在注册连接 */
#define IOCHUB_REGISTERED                       101UL       /* 设备已注册 */
#define IOCHUB_REGISTER_FAIL                    102UL       /* 注册失败（认证第一步）*/
#define IOCHUB_LOGIN_FAIL                       103UL       /* 登录失败（认证第二步） */
#define IOCHUB_CNT_COMM_SER_FAIL                104UL       /* 连接认证服务失败 */
#define IOCHUB_SER_EXCEPTIONS                   105UL       /* 认证服务异常(需要重试) */
#define IOCHUB_CNT_COMM_SER_TIMEOUT             106UL       /* 设备认证超时 */
#define IOCHUB_GET_DEVICEID_FAIL                107UL       /* 获取本机标识失败 */
#define IOCHUB_UPLOAD_NATTYPE_FAIL              108UL       /* 上报NAT类型失败 */
#define IOCHUB_INPUT_IP_FORMAT_ERR              109UL       /* 输入IP及端口格式不正确 */

#define IOCHUB_DEVICE_UNREGISTER                200UL       /* 设备未注册（认证第一步） */
#define IOCHUB_LAN_SCANNING                     201UL       /* 正在进行局域网发现 */
#define IOCHUB_BROADCAST_PORT_USED              202UL       /* 广播端口被占用(广播端口默认:10000) */
#define IOCHUB_UNINIT                           203UL       /* 未初始化IoCHub库 */
#define IOCHUB_LAN_SCANNING_DISABLE             204UL       /* 本机局域网发现不可用 */
#define IOCHUB_LAN_SCAN_OVER                    205UL       /* 局域网扫描结束 */
#define IOCHUB_LIB_INIT_FAIL                    206UL       /* 初始化IoCHub库失败 (新) */

#define IOCHUB_SESSION_EXIT                     300UL       /* 会话已存在 */
#define IOCHUB_LOAD_FULL                        301UL       /* 自身会话连接数已达上限 */
#define IOCHUB_SESSION_OPENING                  302UL       /* 正在打开会话 */
#define IOCHUB_DEVICE_OFFLINE                   303UL       /* 当前设备未认证连接 */
#define IOCHUB_DST_DEVICE_OFFLINE               304UL       /* 目标会话设备不在线 */
#define IOCHUB_DST_DEVICE_LOAD_FULL             305UL       /* 目标设备会话连接数已达上限 */
#define IOCHUB_SESSION_OPEN_FAIL                306UL       /* 打开会话失败 */
#define IOCHUB_SESSION_OPEN_TIMEOUT             307UL       /* 打开会话超时 */
#define IOCHUB_REG_BROADCAST_FAIL               308UL       /* 注册广播失败 */
#define IOCHUB_REGISTERED_BROADCAST             309UL       /* 已经注册广播 */
#define IOCHUB_DSTID_IS_LOCALID                 310UL       /* 目标会话设备ID为本机ID */
#define IOCHUB_SESSION_CLOSE_FAIL               311UL       /* 关闭会话失败 */
#define IOCHUB_LAWLESS_DEVID                    312UL       /* 非法设备NodeID */
#define IOCHUB_DSTDEV_BUSY                      313UL       /* 目标设备正忙(需要重试) */
#define IOCHUB_SESSION_CAPTCHA_FORMAT_ERR       314UL       /* 会话模式会话密钥格式不正确 */
#define IOCHUB_SESSION_CAPTCHA_NOT_MATCH        315UL       /* 会话模式会话密钥不匹配 */
#define IOCHUB_REJECT_SESSION_CONNECT           316UL       /* 拒绝会话连接 */
#define IOCHUB_SESSION_EXCEPTION                317UL       /* 会话异常 */
#define IOCHUB_EXISTS_UNCLOSED_SESSION          318UL       /* 存在未关闭的P2P会话 */

#define IOCHUB_SESSION_NOT_EXIT                 400UL       /* 会话不存在 */
#define IOCHUB_TXBUF_OVERFLOW                   401UL       /* 发送缓冲区溢出 */
#define IOCHUB_RXBUF_OVERFLOW                   402UL       /* 接收缓冲区溢出 */
 
#define IOCHUB_SOCKET_CREATE_FAIL               500UL       /* Socket创建失败 */
#define IOCHUB_SOCKET_CONNECT_TIMEOUT           501UL       /* Socket连接超时 */
#define IOCHUB_SOCKET_CONNECT_FAIL              502UL       /* Socket连接失败 */
#define IOCHUB_SOCKET_READ_ABNORMAL             503UL       /* Socket读异常 */
#define IOCHUB_SOCKET_WRITE_ABNORMAL            504UL       /* Socket写异常 */
#define IOCHUB_SOCKET_CHANNEL_ABNORMAL          505UL       /* Socket通道异常 */
#define IOCHUB_REMOTE_ADDR_UNREACHABLE          506UL       /* 远程地址不可达 */
#define IOCHUB_DEVICE_DISC_ABNORMAL             600UL       /* 设备异常断开 */
#define IOCHUB_UDPUPDATE_REMOTEADDR_ABNORMAL    601UL       /* UDP更新远程连接地址异常 */
#define IOCHUB_REQUEST_PARAM_INVALID            700UL       /* 请求参数无效 */
#define IOCHUB_DEVICE_CONNTED_FORBID_CFGPARAM   701UL       /* 设备已连接,不能进行参数配置 */
#define IOCHUB_HANDLE_INVALID                   702UL       /* 无效句柄 */
#define IOCHUB_PARAM_INVALID                    703UL       /* 配置参数无效或不符合要求 */
#define IOCHUB_MSG_ARRAY_FULL                   704UL       /* 消息队列已满 */


@interface IOCHubCfgModel : NSObject
// 接收缓冲区大小，默认128 * 1024（字节）
@property (nonatomic, assign) UInt32 recvBufferSize;
// 发送缓冲区大小，默认128 * 1024（字节）
@property (nonatomic, assign) UInt32 sendBufferSize;
// 超时时间，默认10 * 1000 （毫秒）
@property (nonatomic, assign) UInt32 socketTimeout;
// 最大连接数量，默认10
@property (nonatomic, assign) UInt32 maxConnSize;
// 每包发送数据大小，默认4 * 1024（字节）,最大值为10 * 1024
@property (nonatomic, assign) UInt32 sendSizePerPackage;
// 窗口大小
@property (nonatomic, assign) UInt32 socketWndSize;
// 窗口应答超时时间，默认100（毫秒）
@property (nonatomic, assign) UInt32 socketWndTimeout;
// 是否开启TLS，默认false（保留，暂不开放）
@property (nonatomic, assign) BOOL supportTLS;
// 是否开启DTLS，默认false（保留，暂不开放）
@property (nonatomic, assign) BOOL supportDTLS;
// 是否开启数据压缩传输，默认false
@property (nonatomic, assign) BOOL supportCompress;
// 是否开启设备锁，默认false
@property (nonatomic, assign) BOOL openDeviceLock;
// 节点属性，0x00：节点终端  0x01：通用网关    0x02：专用接口网关
@property (nonatomic, assign) UInt32 peerProperty;
// 本地网卡，格式如："192.168.111.93,192.168.1.100"，以英文逗号隔开
@property (nonatomic, strong) NSString *localNetwork;

@end


@interface IOCHubAccessModel : NSObject
//访问模式
@property (nonatomic, assign) int modeType;
//密钥
@property (nonatomic, strong) NSString *sessionKey;
//密钥长度
@property (nonatomic, assign) int keySize;

@end
